/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Systme PTT informatis pour microphone de communication 
  Code source pour PIC12F683, dvelopp dans Microchip MPLAB X IDE et compil avec Microchip XC8.
  lectro-Bidouilleur, Dcembre 2022. https://www.youtube.com/lectro-Bidouilleur
  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr
*/

// DIRECTIVES AU COMPILATEUR
#include <xc.h>					// Compilateur utilis par MPLAB X
#include <pic12f683.h>			// Compil pour le PIC12F683
#define _XTAL_FREQ  31000		// La frquence de l'oscillateur  considrer par le compilateur et le simulateur
#define ON 1					   // permet d'utiliser "on" et "off" au lieu de "0" et "1".
#define OFF 0
#define LED_PTT GP0           // Identification des diffrentes broches d'E/S
#define BOUTON_PTT GP1        //      "
#define SORTIE_PTT GP2 
#define BOUTON_PTT_LOCK GP4 
#define LED_PTT_LOCK GP5 


// VARIABLES GLOBALES
unsigned int compte_50ms;

// OCTETS DE CONFIGURATION (FUSIBLES)
#pragma config FOSC = INTOSCIO  	// Bits de slection de l'oscillateur (oscillateur INTOSCIO interne: fonction E/S sur la broche RA4 / OSC2 / CLKOUT, fonction E/S sur RA5/OSC1/CLKIN)
#pragma config WDTE = OFF       	// Bit d'activation de l'horloge de surveillance (WDT dsactiv)
#pragma config PWRTE = OFF      	// Bit d'activation de la minuterie de mise sous tension (PWRT dsactiv)
#pragma config MCLRE = OFF      	// Bit de slection de fonction de broche MCLR (la fonction de broche MCLR configur en entre numrique, MCLR li en interne  VDD)
#pragma config CP = OFF         	// Bit de protection du code (la protection du code de la mmoire du programme est dsactive)
#pragma config CPD = OFF        	// Bit de protection du code de donnes (la protection du code de la mmoire de donnes est dsactive)
#pragma config BOREN = OFF      	// Brown Out Detect (BOR dsactiv)
#pragma config IESO = OFF       	// Bit de commutation externe/interne (le mode de commutation externe interne est dsactiv)
#pragma config FCMEN = OFF      	// Bit activ du moniteur d'horloge de scurit (le moniteur d'horloge de scurit est dsactiv) 


// Routines de minuterie bases sur des boucles
void delay_ms(int delay) 
{
    for (int i = 0; i < delay; i++) __delay_ms(1);
}

// Fonction principale
void main(void) 
{ 
    // Initialisation des registres du micro-contrleur
    PIR1 =       0;          // Dsactiver les interruptions
    IOC =        0b00000000; // Dsactiver les interruptions causes par les broches d'entre
    PIE1 =       0b00000000; // Ne pas permettre les interruptions
    OPTION_REG = 0b00000111; // Pull-ups actives globalement, TMR0 activ.
    WPU =        0b00010010; // Pull-ups individuelles, active sur GP1 et GP4 (boutons-poussoirs)
    OSCCON =     0b00000001; // Selection de l'oscillateur interne, et cadence de 31 Khz
    T1CON =      0b00000101; // Activer TMR1 et slectionner le prescaler division-par-1
    T2CON =      0b01111011; // teindre TMR2 (CCP pas utilis)
    CCP1CON =    0b00000000; // PWM pas utilis
    ADCON0 =     0b10000000; // Ne pas activer l'ADC
    ANSEL =      0b00000000; // Nepas activer la fonction analogique sur les broches
    CMCON0 =     0b00010111; // Dsactiver le comparateur
    CMCON1 =     0b00000000; // Pas utilis car comparateur teint
    VRCON =      0b00100001; // Pas utilis car comparateur teint
    WDTCON =     0b00010100; // Garder le Warchdog teint
    GPIO   =     0b00000000; // Mettre les bons niveaux sur les broches de sortie avant de les activer
    TRISIO =     0b11010010; // Assigner les broches d'entre/sortie comme suit:
								//	GP5-Sortie de la LED du bouton PTT Lock
								//	GP4-Entre du bouton PTT Lock
								//	GP3-Entre uniquement, non utilise 
								//	GP2-Sortie PTT
								//	GP1-Entre du bouton PTT 
								//	GP0-Sortie de la LED du bouton PTT

	delay_ms(250);			 // Dlai de stabilisation de 0.25 seconde

    compte_50ms = 0;
    
    // Boucle principale
    while(1)
    {    
        // Traitement du bouton PTT
        if (!BOUTON_PTT)                        // Bouton PTT enfonc?
        {   // oui        
            SORTIE_PTT = 1;                     // Activer le PTT
            LED_PTT = 1;                        // Allumer la led du PTT
            if (LED_PTT_LOCK) LED_PTT_LOCK = 0; // Si la led PTT Lock est allume, l'teindre
        }
        else if (!LED_PTT_LOCK)                 // Bouton PTT non-enfonc. LED PTT Lock teinte?
        {                                       // Oui
            LED_PTT = 0;                        // teindre la LED du PTT
            SORTIE_PTT = 0;                     // Dsactiver le PTT
        }
        else LED_PTT = 0;                       // Bouton PTT non-enfonc et LED PTT Lock allume, teindre la LED de PTT

        // Traitement du bouton PTT Lock
        if (!BOUTON_PTT_LOCK)                   // Bouton PTT_Lock enfonc?
        {                                       // oui
            if (!SORTIE_PTT)                    // PTT dsactiv?
            {                                   // Oui
                SORTIE_PTT = 1;                 // Activer le PTT
                LED_PTT_LOCK = 1;               // Allumer la LED PTT Lock
            }
            else                                // Bouton PTT_Lock enfonc et PTT dj activ
            {
                if (!LED_PTT)                   // LED de PTT teinte?
                {                               // Oui
                    SORTIE_PTT = 0;             // Relcher le PTT  
                    LED_PTT_LOCK = 0;           // teindre la LED de PTT Lock                       
                }
                else if (!BOUTON_PTT)           // Bouton PTT_Lock enfonc et LED de PTT Allume. Bouton PTT appuy?
                {                               // Oui
                    LED_PTT_LOCK = 1;           // Allumer la LED de PTT Lock
                    while (!BOUTON_PTT);        // Attendre le relchement du bouton PTT
                    LED_PTT = 0;                // teindre la LED de PTT
                }
            }
            while (!BOUTON_PTT_LOCK);           // Attendre le relchement du bouton PTT Lock
        }    

        // Gestion de la minuterie de dpassement de 5 minutes
        __delay_ms(50);                         // Pause de 50ms pour anti-rebond et calcul de minuterie de dpassement
        if (SORTIE_PTT)                         // PTT actif?   
        {                                       // Oui
            compte_50ms++;                      // Incrmenter le compteur de 50ms
            if (compte_50ms >= 6000)            // 5 minutes dpasses?
            {                                   // Oui
                SORTIE_PTT = 0;                 // Relcher le PTT  
                LED_PTT_LOCK = 0;               // teindre la LED de PTT Lock                       
                LED_PTT = 0;                    // teindre la LED de PTT
                compte_50ms = 0;                // Rinitialiser le compteur de 50ms
                while (!BOUTON_PTT);            // Attendre le relchement du bouton PTT
            }           
        }            
        else compte_50ms = 0;                   // PTT inactif. Effacer le compteur de 50ms
   } 
}

